/*                                                              */
/*                                                              */
/*  This Program is free software, distributed under the terms  */
/*  of the GNU General Public License. See the file COPYING or  */
/*  http://www.fsf.org for more information.                    */
/*                                                              */
/*  tunguska       (c) Diane Neisius   (diane_va@yahoo.com)     */
/*                                                              */
/*                                                              */
/*--------------------------------------------------------------*/
/*                                                              */
/*                        Tunguska Program                      */
/*                                                              */
/*                              for                             */
/*                                                              */
/*    Simulation of Breakup of Meteorites in the Atmosphere     */
/*                                                              */
/*                  by Diane Neisius, 15. 01. 2002              */
/*                                                              */
/*                                                              */
/*    Ref.: Hill / Goda,                                        */
/*          The Fragmentation of Small Asteroids                */
/*          in the Atmosphere                                   */
/*          Astr. J. 105(1993) p.1114                           */
/*                                                              */

# include <stdio.h> 
# include <math.h>
# include "numerics.h"

/* Global Constants */

const double pi        = M_PI;     /*        3.1415...          */
const double sigma     = 1.0e-8;   /* SI,    Ablation Parameter */
const double tau       = 1.0e-2;   /* SI,    Lumin. Parameter   */
const double rho_iron  = 8000;     /* kg/m3, Density of Iron    */
const double rho_stone = 3000;     /* kg/m3, Density of Stone   */
const double rho0_air  = 1.293;    /* kg/m3, Density Air Sea Lev*/
const double h0        = 8000;     /* m,     Scale Heigth       */
const double s_iron    = 2.0e+8;   /* Pa,    Mat. Str. Iron     */
const double s_silic   = 4.0e+7;   /* Pa,    Mat. Str. Silicate */
const double s_chond   = 2.0e+7;   /* Pa,    Mat. Str. Chondrite*/
const double g         = 9.81;     /* m/s2,  Grav. Acceleration */
const double c         = 0.5;      /*        Drag Coeff. Sphere */
const double d         = 100000;   /* m,     Scale Dist. Abs.Mag*/
const double f0        = 3.38e-9;  /* J/m2s, Scale Const Abs.Mag*/
const double r_earth   = 6.371e+6; /* m,     Earth Radius       */

/* Global Variables */

double dt, t; 
double r0, v0, rho0, m0, alpha; 
double u[ RUNGE_KUTTA_N ];
int    fragm, separ;
       


/* Ablation and Motion Equations in here */

void ablation_motion( double dy[], double t, double y[] )
   { 
      double m, v, h, w, rd;
      double rho, rho_air, a;
      double vol;

      m  = y[0];
      v  = y[1];
      h  = y[2]; rho_air = rho0_air * exp( - h/h0 );
      w  = y[3];
      rd = y[4]; vol = 4 * pi / 3 * rd * rd * rd;
                 a   = pi * rd * rd;
                 rho = m / vol;

      dy[0] = - sigma * c * a * rho_air * v * v * v;
      dy[1] = - a * c * rho_air * v * v / m  +  g * sin( alpha );
      dy[2] = - v * sin( alpha );
      dy[3] =   v * cos( alpha );

      if ( fragm )
         dy[4] = sqrt( fabs( 3.5 * rho_air / rho0 ) ) * v;
      else
         dy[4] = dy[0] / ( 4 * pi * rd * rd * rho0 );
   }


int main( void )
   {
      int    i;
      FILE * data;
      char   dummy[50];
      char   mat; 
      double m, v, h, w, rd;
      double s, p, rc, old_drdt, t_sep;       
      double rho, rho_air, a;
      double r_frag, r_frl;      
      double d_u[ RUNGE_KUTTA_N ]; 
      double dm_dt, dv_dt, dh_dt, dw_dt, dr_dt;
      double l, l_mag;
      double delta;
      char   x;

      printf(" \n TUNGUSKA -- (c) Diane Neisius (diane_va@yahoo.com)\n\n");

      printf(" This program is free software, distributed under the terms\n");
      printf(" of the GNU General Public License. See the file COPYING or\n");
      printf(" http://www.fsf.org for more information.\n");
       
      if (( data = fopen( "tunguska.in", "r")) == NULL )
        {
           fprintf( stderr, "can't open tunguska.in\n\n" );
           return 1;
        }
      for ( i=0; i<10; i++ )
         if ( fgets( dummy, 45, data ) == NULL )
            fprintf( stderr, "read error \n\n" );

      if ( fgets( dummy, 23, data ) == NULL ) 
         fprintf( stderr, "read error\n\n" );
      if ( fgets( dummy, 25, data ) == NULL ) 
         fprintf( stderr, "read error\n\n" );
      mat = dummy[0];

      if ( fgets( dummy, 23, data ) == NULL ) 
         fprintf( stderr, "read error\n\n" );
      if ( fscanf( data, "%10lf", &r0 ) == 0 ) 
         fprintf( stderr, "read error\n\n" );
      if ( fgets( dummy, 23, data ) == NULL ) 
         fprintf( stderr, "read error\n\n" );

      if ( fgets( dummy, 23, data ) == NULL ) 
         fprintf( stderr, "read error\n\n" );
      if ( fscanf( data, "%10lf", &v0 ) == 0 ) 
         fprintf( stderr, "read error\n\n" );
      if ( fgets( dummy, 23, data ) == NULL ) 
         fprintf( stderr, "read error\n\n" );
  
      if ( fgets( dummy, 23, data ) == NULL ) 
         fprintf( stderr, "read error\n\n" );
      if ( fscanf( data, "%10lf", &alpha ) == 0 ) 
         fprintf( stderr, "read error\n\n" );
      if ( fgets( dummy, 23, data ) == NULL ) 
         fprintf( stderr, "read error\n\n" );
           
      if ( fgets( dummy, 23, data ) == NULL ) 
         fprintf( stderr, "read error\n\n" );
      if ( fscanf( data, "%10lf", &dt ) == 0 ) 
         fprintf( stderr, "read error\n\n" );
      if ( fgets( dummy, 23, data ) == NULL ) 
         fprintf( stderr, "read error\n\n" );
      fclose( data );

      /* input values read successfully,
         test now for sane values         */

      if ( r0 <= 0 )
         { 
           fprintf( stderr, "input error\n\n" );
           return 2;
         }

      if ( v0 < 11200 )
         { 
           fprintf( stderr, "input error\n\n" );
           return 2;
         }

      if ( dt <= 0 )
         { 
           fprintf( stderr, "input error\n\n" );
           return 2;
         }

      if (( alpha <= 0 ) || ( alpha >= 90 ))
         { 
           fprintf( stderr, "input error\n\n" );
           return 2;
         }

      switch ( mat )
         {
            case 'i':
            case 'I': 
               {
                  s    = s_iron;
                  rho0 = rho_iron;
                  printf("\n Iron meteoroid at deg %3.1lf angle"
                         " above horizon\n\n", alpha );
                  break;
               }
            case 's':
            case 'S': 
               {
                  s    = s_silic;
                  rho0 = rho_stone;
                  printf("\n Stone meteoroid at deg %3.1lf angle"
                         " above horizon\n\n", alpha );
                  break;
               }
            case 'c':
            case 'C': 
               {
                  s    = s_chond;
                  rho0 = rho_stone;
                  printf("\n Chondritic stone meteoroid at deg %3.1lf angle"
                         " above horizon\n\n", alpha );
                  break;
               }
            default : 
               {
                  fprintf( stderr, "input error\n\n" );
                  return 2;
               }
         }

      /* values ok, main procedure can start. */
 
      r0 /= 2;  /* note input was "diameter"! */
      alpha *= pi /180; /*   deg -> rad       */

      m0 = 4 * pi / 3 * r0 * r0 * r0 * rho0;

      m = m0;
      v = v0;
      h = 100000;
      w = 0;
      rd = r0;

      rc = 2 * r0;  /* assumed by Hill & Goda */

      fragm = 0;
      separ = 0;


      /*                                       */
      /*              MAIN LOOP                */
      /*                                       */

      printf("    Time(s)    Mass(t) Speed(km/s)"
             " Heigth(km)   Path(km)   Diam.(m)  Abs. Mag.\n");
      printf("+----------+----------+-----------"
             "+----------+----------+----------+----------\n");

      for ( t = 0; h > 0; )
         {
            u[0] = m;
            u[1] = v;
            u[2] = h;
            u[3] = w;
            u[4] = rd;
 
            runge_kutta( u, ablation_motion, t, u, dt );

            t += dt;

            ablation_motion( d_u, t, u );

            m  = u[0]; dm_dt = d_u[0];
            v  = u[1]; dv_dt = d_u[1];
            h  = u[2]; dh_dt = d_u[2];
            w  = u[3]; dw_dt = d_u[3];
            rd = u[4]; dr_dt = d_u[4];

            l     = -tau * dm_dt * v * v / 2;
            l_mag = -2.5 * log10( fabs( l /( 4 * pi * d * d * f0 )));

            printf(" %10.3lf %10.3lf %11.3lf %10.3lf %10.3lf"
                   " %10.3lf %10.3lf\n",
                     t, m/1000, v/1000, h/1000, w/1000, 2*rd, l_mag );

            rho_air = rho0_air * exp( - h/h0 );
            p       = rho_air * v * v;

            /* sinus theorem to adjust alpha by influence of g... */

            delta = asin( sin( pi/2 + alpha ) * g * dt / v );
            alpha += delta;

            /* ...and adjust alpha to curved earth surface. */

            delta = asin( ( dw_dt * dt )/( r_earth + h ) );
            alpha -= delta;

            if ( !fragm && !separ && ( p > s ))
               {
                  fragm = 1;
                  dt /= 2;
                  printf(" *** Initial breakup *** \n");
               }

            if ( ( fragm == 1 ) && ( rd > rc ) && ( p > s ))
               {
                  fragm = 2;
                  printf(" *** Fragmentation continues *** \n");
               }

            if ( fragm && !separ && ( p <= s ))
               {
                  separ = 1;
                  r_frag = rd;
                  r_frl = rd;
                  printf(" *** Fragmentation stops ***\n");
               }

            if ( separ && ( rd >= 2 * r_frag ))
               {
                  separ = 2;
                  fragm = 0;
                  r_frl = rd;
                  dt *= 2;

                  rd = r0 * r0 * r0 / ( r_frag * r_frag ) * m / m0;
                  m  = 4 * pi / 3 * rd * rd * rd * rho0;

                  old_drdt = dr_dt;
                  t_sep = t;

                  printf(" *** Separation, trace biggest piece ***\n");
               }

            if ( v <= 333 )
               {
                  dt = 1.0;
               }

            if ( m <= 0.001 )
               {
                  printf(" *** burned ***\n");
                  return 0;
               }

            if ( h > 100000 )
               {
                  printf(" *** left atmosphere ***\n");
                  return 0;
               }
         }
      printf(" *** Impact *** \n");

      /* main loop completed, calculate some
         values of the "remains"              */

      if ( separ != 2 )
         {
            r_frag = rd;
            r_frl = rd;
            old_drdt = dr_dt;
            t_sep = t;
            rd = r0 * r0 * r0 / ( r_frag * r_frag ) * m / m0;
            m  = 4 * pi / 3 * rd * rd * rd * rho0;
         }

      if ( fragm || separ )
         {
            r_frl += ( t - t_sep ) * old_drdt;
            r_frag = r_frl / sin( alpha );
            printf("\n Debris ellipse %6.3lf x %6.3lf km",
                       2 * r_frag/1000, 2 * r_frl/1000 );
            printf("\n Largest fragment %10.3lf kg\n", m );
         }
      else
         {
           printf("\n Crater Diameter %6.3lf km\n", 40 * rd/1000 );
         }
      return 0;
   }













